/**
   @file Testing the md5 checksum generator.

   @param If no paramater is gven this programm generates the
   checksums if md5test[0-3].dat and compares these with the internal
   stored sums. Otherwise the program generates the md5sums of the
   given file(s) and print these on the screen.
*/

/*
  Copyright (c) Andreas Hofmeier
  (www.an-h.de, www.an-h.de.vu, www.lgut.uni-bremen.de/an-h/)
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <stdio.h>
#include "../lib/md5.h"

/** Buffersize for reading the file to generate a md5sum of it, for
    calculate_md5sum(). */
#define bufsize 2048

// internal sored md5sums of the md5test[0-4].dat files.
char md5sums[4][16] = {
  { 0x9e, 0xb2, 0x86, 0x6e, 0x09, 0xcd, 0x47, 0xed, 0x53, 0x2d, 0x99, 0xb9, 0xc3, 0xcd, 0x4a, 0x86 }, // md5test0.dat
  { 0xd2, 0xb6, 0x5a, 0xe7, 0x29, 0xe7, 0x93, 0x3c, 0x9c, 0xa9, 0x1c, 0x88, 0xdb, 0x7c, 0xd0, 0xfc }, // md5test1.dat
  { 0x4d, 0x3c, 0x9e, 0xff, 0xe9, 0xb4, 0x97, 0x9a, 0x4b, 0xc7, 0xce, 0x2e, 0x45, 0xdf, 0x21, 0xd2 }, // md5test2.dat
  { 0xf5, 0x5c, 0x17, 0x5c, 0x5f, 0x26, 0x98, 0xf9, 0xb2, 0xcc, 0xf7, 0xba, 0x72, 0x01, 0x4b, 0x14 }, // md5test3.dat
};

/**
   Calculate the md5sum of a file

   @param filename (char *) name of the file of which the md5sum has
   to be generated

   @return (char *) pointer to a static memory area in whish the
   md5sum ist stored.
*/
char *calculate_md5sum(char *filename) {
  FILE *f;
  // Structure to store the sub-total md5sum
  struct MD5Context context;
  // memory area to store the md5sum
  static unsigned char md5_prs[16];
  // control variable for size of read block
  int i;
  // buffer for reading the file
  char buf[bufsize];

  // open file
  if ((f = fopen(filename, "r")) == NULL) {
    perror(filename);
    return NULL;
  }
  // init sub-total structure
  MD5Init(&context);
  // read the whole file and calculate the md5sum over it
  while (!feof(f)) {
    i = fread(buf, 1, bufsize, f);
    MD5Update(&context, buf, i);
  }
  fclose(f);
  // build up the md5sum
  MD5Final(md5_prs, &context);
  // return it
  return md5_prs;
}


main(int argc, char *argv[]) {
  int i, j;
  unsigned char *s;
  char fn[128];

  // calculate the md5sum of any file which is given as parameter
  if (argc != 1) {
    for (i = 1; i < argc; i++) {
      if ((s = calculate_md5sum(argv[i])) != NULL) {
	for (j = 0; j < 16; j++){
	  printf("%02x", s[j]);
	}
	printf("  %s\n", argv[i]);
      }
    }
    exit(0);
  }

  // runing normal test
  printf("running %s...\n", argv[0]);

  for (i = 0; i < 4; i++) {
    sprintf(fn, "md5test%d.dat", i);
    printf("  %s...", fn);
    fflush(stdout);
    s = calculate_md5sum(fn);
    if ((strncmp(s, md5sums[i], 16)) != 0) {
      fail("md5sums are not matching");
      exit(1);
    }
    printf(" OK\n", fn);
    fflush(stdout);
  }
  exit(0);
}
